/*
 * Copyright (C) Tue May 25 2004 John Watson
 * john@flagrantdisregard.com
 * http://flagrantdisregard.com/headline
 *
 * $Id: Blog.java,v 1.15 2004/06/24 09:45:11 yafd Exp $
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/*
 * This abstract class represents a blog and the basic actions supported on that
 * blog.  Implementations for specific blogging platforms will extend this
 * abstract class.
 */

// java
import java.awt.Component;
import java.net.*;
import java.io.*;
import javax.swing.*;

// xml
import javax.xml.parsers.*;
import javax.xml.transform.*;
import javax.xml.transform.dom.*;
import javax.xml.transform.stream.*;
import org.w3c.dom.*;
import org.xml.sax.*;

// jedit
import org.gjt.sp.util.Log;

public abstract class Blog extends Component implements Comparable {
    protected static final String APP_KEY = "none";
    
	protected String blogName = "";
	protected String username = "";
	protected String password = "";
    protected String hostName = "";
    protected int port = 0;
    protected String endPoint = "";
    protected String api = BlogType.API_BLOGGER;
    
    public abstract String newPost(String content, boolean publish);
	
    public abstract String editPost(String content, String postId, boolean publish);
    
    public abstract String deletePost(String postId, boolean publish);
    
    public abstract String getRecentPosts(int numberofposts);
    
	public Blog(String blogName, String hostName, int port, String endPoint, String username, String password, String api) {
		this.blogName = blogName.replaceAll("\n", "").replaceAll("\t", "").trim();
        this.hostName = hostName.replaceAll("\n", "").replaceAll("\t", "").trim();
        this.port = port;
        this.endPoint = endPoint.replaceAll("\n", "").replaceAll("\t", "").trim();
		this.username = username.replaceAll("\n", "").replaceAll("\t", "").trim();
		this.password = password.replaceAll("\n", "").replaceAll("\t", "").trim();
        this.api =  api.replaceAll("\n", "").replaceAll("\t", "").trim();
	}
    
	public String toString() {
		return(getName());
	}
	
	public String getName() {
		return(blogName);
	}
	
    public String getUsername() {
        return(username);
    }
    
    public String getPassword() {
        return(password);
    }
    
    public String getHostName() {
        return(hostName);
    }
    
    public int getPort() {
        return(port);
    }
    
    public String getEndPoint() {
        return(endPoint);
    }
    
    public String getAPI() {
        return(api);
    }
    
	public int compareTo(Object o) {
		Blog b = (Blog)o;
		int n = blogName.compareToIgnoreCase(b.blogName);
		if (n < 0) { return(-1); }
		if (n > 0) { return(1); }
		return(0);
	}
    
    protected void promptForPassword() {
        if (password.length() == 0) {
            Object message[] = new Object[2];
            JPasswordField jpf = new JPasswordField();
            message[0] = new JLabel("Password:");
            message[1] = jpf;
            
            JOptionPane.showOptionDialog(
                null,
                message,
                "What is your password?",
                JOptionPane.OK_CANCEL_OPTION,
                JOptionPane.QUESTION_MESSAGE,
                null,
                null,
                null);
                
            password = String.valueOf(jpf.getPassword());
        }
    }
    
    protected String getFault(String response) {
        // Load and parse xml response
        Document doc = null;
        DocumentBuilder db = null;
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        factory.setValidating(false);
        
        try {
            db = factory.newDocumentBuilder();
            doc = db.parse(new ByteArrayInputStream(response.getBytes()));
        } catch (IOException ex) {
            JOptionPane.showMessageDialog(null, "An error occurred trying to\nparse the response");
            Log.log(Log.MESSAGE, Headline.class, "Could not parse response: " + ex);
            return("Could not parse response: " + ex);
        } catch (ParserConfigurationException ex) {
            JOptionPane.showMessageDialog(null, "An error occurred trying to\nparse the response");
            Log.log(Log.MESSAGE, Headline.class, "Could not parse response: " + ex);
            return("Could not parse response: " + ex);
        } catch (SAXException ex) {
            JOptionPane.showMessageDialog(null, "An error occurred trying to\nparse the response");
            Log.log(Log.MESSAGE, Headline.class, "Could not parse response: " + ex);
            return("Could not parse response: " + ex);
        }
        
        NodeList fault = doc.getDocumentElement().getElementsByTagName("fault");
        if (fault.getLength() > 0) {
            return(XMLHelper.NodeText(fault.item(0)));
        }
        
        return(null);
    }
    
    protected String getAvailableBlogs() {
        StringBuffer request = new StringBuffer();
        String response;
        String fault;
        String blogId = null;
        
        class UserBlog {
            public String blogId;
            public String blogName;
            
            public UserBlog(String blogId, String blogName) {
                this.blogId = blogId;
                this.blogName = blogName;
            }
            
            public UserBlog() {
                this.blogId = null;
                this.blogName = null;
            }
            
            public String toString() {
                return(blogName);
            }
        }
        
        // Some systems (like http://xmlrpc.free-conversant.com/docs/bloggerAPI) don't
        // support the concept of multiple blogs per user.  In which case, this call needs to
        // fail gracefully and the next call needs to attempt to do the Right thing.
        request.append("<?xml version=\"1.0\"?>");
        request.append("<methodCall>");
        request.append("<methodName>blogger.getUsersBlogs</methodName>");
        request.append("<params>");
        request.append("<param><value><string>").append(APP_KEY).append("</string></value></param>");
        request.append("<param><value><string>").append(username).append("</string></value></param>");
        request.append("<param><value><string>").append(password).append("</string></value></param>");
        request.append("</params>");
        request.append("</methodCall>");
        response = connect(request.toString());
        if (response == null) {
            return(null);
        }
        fault = getFault(response);
        if (fault != null) {
            JOptionPane.showMessageDialog(null, "The server returned an error:" + fault + "\nSee activity log for details.");
            Log.log(Log.MESSAGE, Headline.class, "Server fault: " + response);
            return(null);
        }
        
        // Load and parse xml response
        Document doc = null;
        DocumentBuilder db = null;
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        factory.setValidating(false);
        
        try {
            db = factory.newDocumentBuilder();
            doc = db.parse(new ByteArrayInputStream(response.getBytes()));
        } catch (IOException ex) {
            JOptionPane.showMessageDialog(null, "An error occurred trying to\nparse the response from the server");
            Log.log(Log.MESSAGE, Headline.class, "Could not parse response: " + ex);
            return(null);
        } catch (ParserConfigurationException ex) {
            JOptionPane.showMessageDialog(null, "An error occurred trying to\nparse the response from the server");
            Log.log(Log.MESSAGE, Headline.class, "Could not parse response: " + ex);
            return(null);
        } catch (SAXException ex) {
            JOptionPane.showMessageDialog(null, "An error occurred trying to\nparse the response from the server");
            Log.log(Log.MESSAGE, Headline.class, "Could not parse response: " + ex);
            return(null);
        }
        
        UserBlog[] ub = null;
        
        try {
            NodeList blogStruct = doc.getDocumentElement().getElementsByTagName("struct");
            if (blogStruct.getLength() <= 0) {
                JOptionPane.showMessageDialog(null, "No blogs.");
                return(null);
            }
            
            ub = new UserBlog[blogStruct.getLength()];
            
            for (int i=0; i<blogStruct.getLength(); i++) {
                NodeList member = blogStruct.item(i).getChildNodes();
                
                /*
                <struct>
                    <member>
                        <name>blogid</name>
                        <value>7012667</value>
                    </member>
                    <member>
                        <name>blogName</name>
                        <value>flagrantdisregard</value>
                    </member>
                    ...
                </struct>
                */
                
                ub[i] = new UserBlog();
                
                for(int j=0; j<member.getLength(); j++) {
                    NodeList children = member.item(j).getChildNodes();
                    
                    String name = XMLHelper.NodeText(children.item(0));
                    String value = XMLHelper.NodeText(children.item(1));
                    
                    if (name.compareTo("blogName") == 0) {
                        ub[i].blogName = value;
                    }
                    
                    if (name.compareTo("blogid") == 0) {
                        ub[i].blogId = value;
                    }
                    
                    if (ub[i].blogName != null && ub[i].blogId != null) {
                        break;
                    }
                }
            }
        } catch (Exception ex) {
            JOptionPane.showMessageDialog(null, "An error occurred while parsing the blog list");
            Log.log(Log.MESSAGE, Headline.class, "An error occurred while parsing the blog list: " + ex);
            return(null);
        }
        
        if (ub != null) {
            // If only one blog is available, use that one without prompting
            if (ub.length == 1) {
                blogId = ub[0].blogId;
            } else {
                UserBlog b = (UserBlog)JOptionPane.showInputDialog(
                    null,
                    "Blog: ",
                    "Choose a blog",
                    JOptionPane.PLAIN_MESSAGE,
                    null,
                    ub,
                    null);
                
                if (b != null) { blogId = b.blogId; }
            }
        } else {
            JOptionPane.showMessageDialog(null, "No blogs found.");
        }
        
        return(blogId);
    }
    
    protected String connect(String request) {
        String response;
        URL url = null;
        
        // Setup connection url
        try {
            url = new URL("http", hostName, port, endPoint);
        } catch (MalformedURLException ex) {
            JOptionPane.showMessageDialog(null, "Invalid connection parameters");
            Log.log(Log.MESSAGE, Headline.class, "Invalid connection parameters: " + ex);
            return(null);
        }
        
        try {
            // Connect
            Log.log(Log.MESSAGE, Headline.class, "Connecting to blog " + url + "...");
            HttpURLConnection conn = (HttpURLConnection)url.openConnection();
            conn.setRequestProperty("Content-Length", String.valueOf(request.length()));
            conn.setRequestProperty("Content-Type","text/xml; charset=utf-8");
            conn.setRequestMethod("POST"); // do HTTP POST
            conn.setDoInput(true); // want input
            conn.setDoOutput(true); // want output
            
            // Send data
            Log.log(Log.MESSAGE, Headline.class, "Sending blog data... " + request);
            DataOutputStream output = new DataOutputStream(conn.getOutputStream());
            output.writeBytes(request);
            output.flush();
            output.close();
            
            // Read response
            BufferedReader input = new BufferedReader(new InputStreamReader(conn.getInputStream()));
            String s;
            response = "";
            while ((s = input.readLine()) != null) {
                response += s;
            }
            input.close();
            Log.log(Log.MESSAGE, Headline.class, "Response: " + response);
            
            return(response);
        } catch (IOException ex) {
            JOptionPane.showMessageDialog(null, "Could not connect to server");
            Log.log(Log.MESSAGE, Headline.class, "Could not connect to server: " + ex);
            return(null);
        }
    }
}

